@@ -7,17 +7,12 @@ from toolz import concat
 import numpy as np
 from . import numpy_compat as npcompat
 
-from ..compatibility import getargspec, Container, Iterable, Sequence
+from ..compatibility import Container, Iterable, Sequence
 from ..core import flatten
 from ..utils import ignoring
 
 from numbers import Integral
 
-try:
-    from numpy import broadcast_to
-except ImportError:  # pragma: no cover
-    broadcast_to = npcompat.broadcast_to
-
 try:
     from numpy import take_along_axis
 except ImportError:  # pragma: no cover
@@ -29,9 +24,6 @@ def keepdims_wrapper(a_callable):
     A wrapper for functions that don't provide keepdims to ensure that they do.
     """
 
-    if "keepdims" in getargspec(a_callable).args:
-        return a_callable
-
     @wraps(a_callable)
     def keepdims_wrapped_callable(x, axis=None, keepdims=None, *args, **kwargs):
         r = a_callable(x, axis=axis, *args, **kwargs)
@@ -62,18 +54,18 @@ def keepdims_wrapper(a_callable):
 
 
 # Wrap NumPy functions to ensure they provide keepdims.
-sum = keepdims_wrapper(np.sum)
-prod = keepdims_wrapper(np.prod)
-min = keepdims_wrapper(np.min)
-max = keepdims_wrapper(np.max)
+sum = np.sum
+prod = np.prod
+min = np.min
+max = np.max
 argmin = keepdims_wrapper(np.argmin)
 nanargmin = keepdims_wrapper(np.nanargmin)
 argmax = keepdims_wrapper(np.argmax)
 nanargmax = keepdims_wrapper(np.nanargmax)
-any = keepdims_wrapper(np.any)
-all = keepdims_wrapper(np.all)
-nansum = keepdims_wrapper(np.nansum)
-nanprod = keepdims_wrapper(np.nanprod)
+any = np.any
+all = np.all
+nansum = np.nansum
+nanprod = np.nanprod
 
 try:
     from numpy import nancumprod, nancumsum
@@ -81,25 +73,22 @@ except ImportError:  # pragma: no cover
     nancumprod = npcompat.nancumprod
     nancumsum = npcompat.nancumsum
 
-nancumprod = keepdims_wrapper(nancumprod)
-nancumsum = keepdims_wrapper(nancumsum)
-
-nanmin = keepdims_wrapper(np.nanmin)
-nanmax = keepdims_wrapper(np.nanmax)
-mean = keepdims_wrapper(np.mean)
+nanmin = np.nanmin
+nanmax = np.nanmax
+mean = np.mean
 
 with ignoring(AttributeError):
-    nanmean = keepdims_wrapper(np.nanmean)
+    nanmean = np.nanmean
 
-var = keepdims_wrapper(np.var)
+var = np.var
 
 with ignoring(AttributeError):
-    nanvar = keepdims_wrapper(np.nanvar)
+    nanvar = np.nanvar
 
-std = keepdims_wrapper(np.std)
+std = np.std
 
 with ignoring(AttributeError):
-    nanstd = keepdims_wrapper(np.nanstd)
+    nanstd = np.nanstd
 
 
 def coarsen(reduction, x, axes, trim_excess=False):
@@ -237,7 +226,7 @@ def argtopk(a_plus_idx, k, axis, keepdims):
     if isinstance(a_plus_idx, list):
         a_plus_idx = list(flatten(a_plus_idx))
         a = np.concatenate([ai for ai, _ in a_plus_idx], axis)
-        idx = np.concatenate([broadcast_to(idxi, ai.shape)
+        idx = np.concatenate([np.broadcast_to(idxi, ai.shape)
                               for ai, idxi in a_plus_idx], axis)
     else:
         a, idx = a_plus_idx
@@ -294,7 +283,7 @@ def einsum(*operands, **kwargs):
     dtype = kwargs.pop('kernel_dtype')
     chunk = np.einsum(subscripts, *operands, dtype=dtype, **kwargs)
 
-    # Avoid concatenate=True in atop by adding 1's
+    # Avoid concatenate=True in blockwise by adding 1's
     # for the contracted dimensions
     return chunk.reshape(chunk.shape + (1,) * ncontract_inds)
 
