@@ -8,8 +8,8 @@ import numpy as np
 
 from .core import (normalize_chunks, Array, slices_from_chunks, asarray,
                    broadcast_shapes, broadcast_to)
-from .. import sharedict
 from ..base import tokenize
+from ..highlevelgraph import HighLevelGraph
 from ..utils import ignoring, random_state_data, skip_doctest
 
 
@@ -67,7 +67,7 @@ class RandomState(object):
         extra_chunks should be a chunks tuple to append to the end of chunks
         """
         size = kwargs.pop('size', None)
-        chunks = kwargs.pop('chunks')
+        chunks = kwargs.pop('chunks', 'auto')
         extra_chunks = kwargs.pop('extra_chunks', ())
 
         if size is not None and not isinstance(size, (tuple, list)):
@@ -99,10 +99,12 @@ class RandomState(object):
         dsks = []
         lookup = {}
         small_args = []
+        dependencies = []
         for i, ar in enumerate(args):
             if isinstance(ar, (np.ndarray, Array)):
                 res = _broadcast_any(ar, size, chunks)
                 if isinstance(res, Array):
+                    dependencies.append(res)
                     dsks.append(res.dask)
                     lookup[i] = res.name
                 elif isinstance(res, np.ndarray):
@@ -118,6 +120,7 @@ class RandomState(object):
             if isinstance(ar, (np.ndarray, Array)):
                 res = _broadcast_any(ar, size, chunks)
                 if isinstance(res, Array):
+                    dependencies.append(res)
                     dsks.append(res.dask)
                     lookup[key] = res.name
                 elif isinstance(res, np.ndarray):
@@ -141,6 +144,7 @@ class RandomState(object):
         keys = product([name], *([range(len(bd)) for bd in chunks] +
                                  [[0]] * len(extra_chunks)))
         blocks = product(*[range(len(bd)) for bd in chunks])
+
         vals = []
         for seed, size, slc, block in zip(seeds, sizes, slices, blocks):
             arg = []
@@ -149,6 +153,7 @@ class RandomState(object):
                     arg.append(ar)
                 else:
                     if isinstance(ar, Array):
+                        dependencies.append(ar)
                         arg.append((lookup[i], ) + block)
                     else:   # np.ndarray
                         arg.append((getitem, lookup[i], slc))
@@ -158,30 +163,35 @@ class RandomState(object):
                     kwrg[k] = ar
                 else:
                     if isinstance(ar, Array):
+                        dependencies.append(ar)
                         kwrg[k] = (lookup[k], ) + block
                     else:   # np.ndarray
                         kwrg[k] = (getitem, lookup[k], slc)
             vals.append((_apply_random, self._RandomState, funcname, seed, size, arg, kwrg))
+
         dsk.update(dict(zip(keys, vals)))
-        dsk = sharedict.merge((name, dsk), *dsks)
-        return Array(dsk, name, chunks + extra_chunks, dtype=dtype)
+
+        graph = HighLevelGraph.from_collections(
+            name, dsk, dependencies=dependencies,
+        )
+        return Array(graph, name, chunks + extra_chunks, dtype=dtype)
 
     @doc_wraps(np.random.RandomState.beta)
-    def beta(self, a, b, size=None, chunks=None):
+    def beta(self, a, b, size=None, chunks="auto"):
         return self._wrap('beta', a, b, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.binomial)
-    def binomial(self, n, p, size=None, chunks=None):
+    def binomial(self, n, p, size=None, chunks="auto"):
         return self._wrap('binomial', n, p, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.chisquare)
-    def chisquare(self, df, size=None, chunks=None):
+    def chisquare(self, df, size=None, chunks="auto"):
         return self._wrap('chisquare', df, size=size, chunks=chunks)
 
     with ignoring(AttributeError):
         @doc_wraps(np.random.RandomState.choice)
-        def choice(self, a, size=None, replace=True, p=None, chunks=None):
-            dsks = []
+        def choice(self, a, size=None, replace=True, p=None, chunks="auto"):
+            dependencies = []
             # Normalize and validate `a`
             if isinstance(a, Integral):
                 # On windows the output dtype differs if p is provided or
@@ -197,7 +207,7 @@ class RandomState(object):
                 if a.ndim != 1:
                     raise ValueError("a must be one dimensional")
                 len_a = len(a)
-                dsks.append(a.dask)
+                dependencies.append(a)
                 a = a.__dask_keys__()[0]
 
             # Normalize and validate `p`
@@ -217,7 +227,7 @@ class RandomState(object):
                 if len(p) != len_a:
                     raise ValueError("a and p must have the same size")
 
-                dsks.append(p.dask)
+                dependencies.append(p)
                 p = p.__dask_keys__()[0]
 
             if size is None:
@@ -240,150 +250,150 @@ class RandomState(object):
             dsk = {k: (_choice, state, a, size, replace, p) for
                    k, state, size in zip(keys, state_data, sizes)}
 
-            return Array(sharedict.merge((name, dsk), *dsks),
-                         name, chunks, dtype=dtype)
+            graph = HighLevelGraph.from_collections(name, dsk, dependencies=dependencies)
+            return Array(graph, name, chunks, dtype=dtype)
 
     # @doc_wraps(np.random.RandomState.dirichlet)
-    # def dirichlet(self, alpha, size=None, chunks=None):
+    # def dirichlet(self, alpha, size=None, chunks="auto"):
 
     @doc_wraps(np.random.RandomState.exponential)
-    def exponential(self, scale=1.0, size=None, chunks=None):
+    def exponential(self, scale=1.0, size=None, chunks="auto"):
         return self._wrap('exponential', scale, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.f)
-    def f(self, dfnum, dfden, size=None, chunks=None):
+    def f(self, dfnum, dfden, size=None, chunks="auto"):
         return self._wrap('f', dfnum, dfden, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.gamma)
-    def gamma(self, shape, scale=1.0, size=None, chunks=None):
+    def gamma(self, shape, scale=1.0, size=None, chunks="auto"):
         return self._wrap('gamma', shape, scale, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.geometric)
-    def geometric(self, p, size=None, chunks=None):
+    def geometric(self, p, size=None, chunks="auto"):
         return self._wrap('geometric', p, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.gumbel)
-    def gumbel(self, loc=0.0, scale=1.0, size=None, chunks=None):
+    def gumbel(self, loc=0.0, scale=1.0, size=None, chunks="auto"):
         return self._wrap('gumbel', loc, scale, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.hypergeometric)
-    def hypergeometric(self, ngood, nbad, nsample, size=None, chunks=None):
+    def hypergeometric(self, ngood, nbad, nsample, size=None, chunks="auto"):
         return self._wrap('hypergeometric', ngood, nbad, nsample,
                           size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.laplace)
-    def laplace(self, loc=0.0, scale=1.0, size=None, chunks=None):
+    def laplace(self, loc=0.0, scale=1.0, size=None, chunks="auto"):
         return self._wrap('laplace', loc, scale, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.logistic)
-    def logistic(self, loc=0.0, scale=1.0, size=None, chunks=None):
+    def logistic(self, loc=0.0, scale=1.0, size=None, chunks="auto"):
         return self._wrap('logistic', loc, scale, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.lognormal)
-    def lognormal(self, mean=0.0, sigma=1.0, size=None, chunks=None):
+    def lognormal(self, mean=0.0, sigma=1.0, size=None, chunks="auto"):
         return self._wrap('lognormal', mean, sigma, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.logseries)
-    def logseries(self, p, size=None, chunks=None):
+    def logseries(self, p, size=None, chunks="auto"):
         return self._wrap('logseries', p, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.multinomial)
-    def multinomial(self, n, pvals, size=None, chunks=None):
+    def multinomial(self, n, pvals, size=None, chunks="auto"):
         return self._wrap('multinomial', n, pvals, size=size, chunks=chunks,
                           extra_chunks=((len(pvals),),))
 
     @doc_wraps(np.random.RandomState.negative_binomial)
-    def negative_binomial(self, n, p, size=None, chunks=None):
+    def negative_binomial(self, n, p, size=None, chunks="auto"):
         return self._wrap('negative_binomial', n, p, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.noncentral_chisquare)
-    def noncentral_chisquare(self, df, nonc, size=None, chunks=None):
+    def noncentral_chisquare(self, df, nonc, size=None, chunks="auto"):
         return self._wrap('noncentral_chisquare', df, nonc, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.noncentral_f)
-    def noncentral_f(self, dfnum, dfden, nonc,  size=None, chunks=None):
+    def noncentral_f(self, dfnum, dfden, nonc,  size=None, chunks="auto"):
         return self._wrap('noncentral_f', dfnum, dfden, nonc, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.normal)
-    def normal(self, loc=0.0, scale=1.0, size=None, chunks=None):
+    def normal(self, loc=0.0, scale=1.0, size=None, chunks="auto"):
         return self._wrap('normal', loc, scale, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.pareto)
-    def pareto(self, a, size=None, chunks=None):
+    def pareto(self, a, size=None, chunks="auto"):
         return self._wrap('pareto', a, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.poisson)
-    def poisson(self, lam=1.0, size=None, chunks=None):
+    def poisson(self, lam=1.0, size=None, chunks="auto"):
         return self._wrap('poisson', lam, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.power)
-    def power(self, a, size=None, chunks=None):
+    def power(self, a, size=None, chunks="auto"):
         return self._wrap('power', a, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.randint)
-    def randint(self, low, high=None, size=None, chunks=None):
+    def randint(self, low, high=None, size=None, chunks="auto"):
         return self._wrap('randint', low, high, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.random_integers)
-    def random_integers(self, low, high=None, size=None, chunks=None):
+    def random_integers(self, low, high=None, size=None, chunks="auto"):
         return self._wrap('random_integers', low, high, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.random_sample)
-    def random_sample(self, size=None, chunks=None):
+    def random_sample(self, size=None, chunks="auto"):
         return self._wrap('random_sample', size=size, chunks=chunks)
 
     random = random_sample
 
     @doc_wraps(np.random.RandomState.rayleigh)
-    def rayleigh(self, scale=1.0, size=None, chunks=None):
+    def rayleigh(self, scale=1.0, size=None, chunks="auto"):
         return self._wrap('rayleigh', scale, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.standard_cauchy)
-    def standard_cauchy(self, size=None, chunks=None):
+    def standard_cauchy(self, size=None, chunks="auto"):
         return self._wrap('standard_cauchy', size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.standard_exponential)
-    def standard_exponential(self, size=None, chunks=None):
+    def standard_exponential(self, size=None, chunks="auto"):
         return self._wrap('standard_exponential', size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.standard_gamma)
-    def standard_gamma(self, shape, size=None, chunks=None):
+    def standard_gamma(self, shape, size=None, chunks="auto"):
         return self._wrap('standard_gamma', shape, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.standard_normal)
-    def standard_normal(self, size=None, chunks=None):
+    def standard_normal(self, size=None, chunks="auto"):
         return self._wrap('standard_normal', size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.standard_t)
-    def standard_t(self, df, size=None, chunks=None):
+    def standard_t(self, df, size=None, chunks="auto"):
         return self._wrap('standard_t', df, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.tomaxint)
-    def tomaxint(self, size=None, chunks=None):
+    def tomaxint(self, size=None, chunks="auto"):
         return self._wrap('tomaxint', size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.triangular)
-    def triangular(self, left, mode, right, size=None, chunks=None):
+    def triangular(self, left, mode, right, size=None, chunks="auto"):
         return self._wrap('triangular', left, mode, right, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.uniform)
-    def uniform(self, low=0.0, high=1.0, size=None, chunks=None):
+    def uniform(self, low=0.0, high=1.0, size=None, chunks="auto"):
         return self._wrap('uniform', low, high, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.vonmises)
-    def vonmises(self, mu, kappa, size=None, chunks=None):
+    def vonmises(self, mu, kappa, size=None, chunks="auto"):
         return self._wrap('vonmises', mu, kappa, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.wald)
-    def wald(self, mean, scale, size=None, chunks=None):
+    def wald(self, mean, scale, size=None, chunks="auto"):
         return self._wrap('wald', mean, scale, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.weibull)
-    def weibull(self, a, size=None, chunks=None):
+    def weibull(self, a, size=None, chunks="auto"):
         return self._wrap('weibull', a, size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.zipf)
-    def zipf(self, a, size=None, chunks=None):
+    def zipf(self, a, size=None, chunks="auto"):
         return self._wrap('zipf', a, size=size, chunks=chunks)
 
 
