@@ -11,7 +11,9 @@ from numbers import Integral
 from toolz import compose, partition_all, get, accumulate, pluck
 
 from . import chunk
-from .core import _concatenate2, Array, atop, lol_tuples, handle_out
+from .core import _concatenate2, Array, handle_out
+from .blockwise import blockwise
+from ..blockwise import lol_tuples
 from .creation import arange
 from .ufunc import sqrt
 from .utils import validate_axis
@@ -19,8 +21,9 @@ from .wrap import zeros, ones
 from .numpy_compat import ma_divide, divide as np_divide
 from ..compatibility import getargspec, builtins
 from ..base import tokenize
+from ..highlevelgraph import HighLevelGraph
 from ..utils import ignoring, funcname, Dispatch
-from .. import config, sharedict
+from .. import config
 
 # Generic functions to support chunks of different types
 empty_lookup = Dispatch('empty')
@@ -139,7 +142,7 @@ def reduction(x, chunk, aggregate, axis=None, keepdims=False, dtype=None,
     # Map chunk across all blocks
     inds = tuple(range(x.ndim))
     # The dtype of `tmp` doesn't actually matter, and may be incorrect.
-    tmp = atop(chunk, inds, x, inds, axis=axis, keepdims=True, dtype=x.dtype)
+    tmp = blockwise(chunk, inds, x, inds, axis=axis, keepdims=True, dtype=x.dtype)
     tmp._chunks = tuple((output_size, ) * len(c) if i in axis else c
                         for i, c in enumerate(tmp.chunks))
     result = _tree_reduce(tmp, aggregate, axis, keepdims, dtype, split_every,
@@ -219,7 +222,8 @@ def partial_reduce(func, x, split_every, keepdims=False, dtype=None, name=None):
         dummy = dict(i for i in enumerate(p) if i[0] not in decided)
         g = lol_tuples((x.name,), range(x.ndim), decided, dummy)
         dsk[(name,) + k] = (func, g)
-    return Array(sharedict.merge(x.dask, (name, dsk)), name, out_chunks, dtype=dtype)
+    graph = HighLevelGraph.from_collections(name, dsk, dependencies=[x])
+    return Array(graph, name, out_chunks, dtype=dtype)
 
 
 @wraps(chunk.sum)
@@ -228,8 +232,9 @@ def sum(a, axis=None, dtype=None, keepdims=False, split_every=None, out=None):
         dt = dtype
     else:
         dt = getattr(np.empty((1,), dtype=a.dtype).sum(), 'dtype', object)
-    return reduction(a, chunk.sum, chunk.sum, axis=axis, keepdims=keepdims,
-                     dtype=dt, split_every=split_every, out=out)
+    result = reduction(a, chunk.sum, chunk.sum, axis=axis, keepdims=keepdims,
+                       dtype=dt, split_every=split_every, out=out)
+    return result
 
 
 @wraps(chunk.prod)
@@ -614,6 +619,15 @@ def arg_reduction(x, chunk, combine, agg, axis=None, split_every=None, out=None)
         raise TypeError("axis must be either `None` or int, "
                         "got '{0}'".format(axis))
 
+    for ax in axis:
+        chunks = x.chunks[ax]
+        if len(chunks) > 1 and np.isnan(chunks).any():
+            raise ValueError(
+                "Arg-reductions do not work with arrays that have "
+                "unknown chunksizes.  At some point in your computation "
+                "this array lost chunking information"
+            )
+
     # Map chunk across all blocks
     name = 'arg-reduce-{0}'.format(tokenize(axis, x, chunk,
                                             combine, split_every))
@@ -631,7 +645,8 @@ def arg_reduction(x, chunk, combine, agg, axis=None, split_every=None, out=None)
     dsk = dict(((name,) + k, (chunk, (old,) + k, axis, off)) for (k, off)
                in zip(keys, offset_info))
     # The dtype of `tmp` doesn't actually matter, just need to provide something
-    tmp = Array(sharedict.merge(x.dask, (name, dsk)), name, chunks, dtype=x.dtype)
+    graph = HighLevelGraph.from_collections(name, dsk, dependencies=[x])
+    tmp = Array(graph, name, chunks, dtype=x.dtype)
     dtype = np.argmin([1]).dtype
     result = _tree_reduce(tmp, agg, axis, False, dtype, split_every, combine)
     return handle_out(out, result)
@@ -741,7 +756,8 @@ def cumreduction(func, binop, ident, x, axis=None, dtype=None, out=None):
                                       (operator.getitem, (m.name,) + old, slc))
             dsk[(name,) + ind] = (binop, this_slice, (m.name,) + ind)
 
-    result = Array(sharedict.merge(m.dask, (name, dsk)), name, x.chunks, m.dtype)
+    graph = HighLevelGraph.from_collections(name, dsk, dependencies=[m])
+    result = Array(graph, name, x.chunks, m.dtype)
     return handle_out(out, result)
 
 
