@@ -9,7 +9,7 @@ try:
 except ImportError:
     yaml = None
 
-from .compatibility import makedirs, builtins, Mapping
+from .compatibility import makedirs, builtins, Mapping, string_types
 
 
 no_default = '__no_default__'
@@ -68,10 +68,12 @@ def update(old, new, priority='new'):
     dask.config.merge
     """
     for k, v in new.items():
-        if k not in old and type(v) is dict:
+        if k not in old and isinstance(v, Mapping):
             old[k] = {}
 
-        if type(v) is dict:
+        if isinstance(v, Mapping):
+            if old[k] is None:
+                old[k] = {}
             update(old[k], v, priority=priority)
         else:
             if priority == 'new' or k not in old:
@@ -102,6 +104,38 @@ def merge(*dicts):
     return result
 
 
+def normalize_key(key):
+    """ Replaces underscores with hyphens in string keys
+
+    Parameters
+    ----------
+    key : string, int, or float
+        Key to assign.
+    """
+    if isinstance(key, string_types):
+        key = key.replace('_', '-')
+    return key
+
+
+def normalize_nested_keys(config):
+    """ Replaces underscores with hyphens for keys for a nested Mapping
+
+    Examples
+    --------
+    >>> a = {'x': 1, 'y_1': {'a_2': 2}}
+    >>> normalize_nested_keys(a)
+    {'x': 1, 'y-1': {'a-2': 2}}
+    """
+    config_norm = {}
+    for key, value in config.items():
+        if isinstance(value, Mapping):
+            value = normalize_nested_keys(value)
+        key_norm = normalize_key(key)
+        config_norm[key_norm] = value
+
+    return config_norm
+
+
 def collect_yaml(paths=paths):
     """ Collect configuration from yaml files
 
@@ -113,11 +147,15 @@ def collect_yaml(paths=paths):
     for path in paths:
         if os.path.exists(path):
             if os.path.isdir(path):
-                file_paths.extend(sorted([
-                    os.path.join(path, p)
-                    for p in os.listdir(path)
-                    if os.path.splitext(p)[1].lower() in ('.json', '.yaml', '.yml')
-                ]))
+                try:
+                    file_paths.extend(sorted([
+                        os.path.join(path, p)
+                        for p in os.listdir(path)
+                        if os.path.splitext(p)[1].lower() in ('.json', '.yaml', '.yml')
+                    ]))
+                except OSError:
+                    # Ignore permission errors
+                    pass
             else:
                 file_paths.append(path)
 
@@ -125,9 +163,14 @@ def collect_yaml(paths=paths):
 
     # Parse yaml files
     for path in file_paths:
-        with open(path) as f:
-            data = yaml.load(f.read()) or {}
-            configs.append(data)
+        try:
+            with open(path) as f:
+                data = yaml.load(f.read()) or {}
+                data = normalize_nested_keys(data)
+                configs.append(data)
+        except (OSError, IOError):
+            # Ignore permission errors
+            pass
 
     return configs
 
@@ -149,7 +192,8 @@ def collect_env(env=None):
     d = {}
     for name, value in env.items():
         if name.startswith('DASK_'):
-            varname = name[5:].lower().replace('__', '.').replace('_', '-')
+            varname = name[5:].lower().replace('__', '.')
+            varname = normalize_key(varname)
             try:
                 d[varname] = ast.literal_eval(value)
             except (SyntaxError, ValueError):
@@ -285,16 +329,16 @@ class set(object):
         path: List[str]
             Used internally to hold the path of old values
         """
+        key = normalize_key(keys[0])
         if len(keys) == 1:
             if old is not None:
-                path_key = tuple(path + [keys[0]])
-                if keys[0] in d:
-                    old[path_key] = d[keys[0]]
+                path_key = tuple(path + [key])
+                if key in d:
+                    old[path_key] = d[key]
                 else:
                     old[path_key] = '--delete--'
-            d[keys[0]] = value
+            d[key] = value
         else:
-            key = keys[0]
             if key not in d:
                 d[key] = {}
                 if old is not None:
@@ -392,6 +436,7 @@ def get(key, default=no_default, config=config):
     keys = key.split('.')
     result = config
     for k in keys:
+        k = normalize_key(k)
         try:
             result = result[k]
         except (TypeError, IndexError, KeyError):
