@@ -3,7 +3,6 @@ from __future__ import absolute_import, division, print_function
 from datetime import datetime
 from collections import defaultdict
 
-from toolz import merge
 import bisect
 import numpy as np
 import pandas as pd
@@ -11,11 +10,10 @@ import pandas as pd
 from .core import new_dd_object, Series
 from . import methods
 from ..base import tokenize
+from ..highlevelgraph import HighLevelGraph
 
 
-class _LocIndexer(object):
-    """ Helper class for the .loc accessor """
-
+class _IndexerBase(object):
     def __init__(self, obj):
         self.obj = obj
 
@@ -23,6 +21,10 @@ class _LocIndexer(object):
     def _name(self):
         return self.obj._name
 
+    @property
+    def _meta_indexer(self):
+        raise NotImplementedError
+
     def _make_meta(self, iindexer, cindexer):
         """
         get metadata
@@ -30,7 +32,46 @@ class _LocIndexer(object):
         if cindexer is None:
             return self.obj
         else:
-            return self.obj._meta.loc[:, cindexer]
+            return self._meta_indexer[:, cindexer]
+
+
+class _iLocIndexer(_IndexerBase):
+
+    @property
+    def _meta_indexer(self):
+        return self.obj._meta.iloc
+
+    def __getitem__(self, key):
+
+        # dataframe
+        msg = ("'DataFrame.iloc' only supports selecting columns. "
+               "It must be used like 'df.iloc[:, column_indexer]'.")
+        if not isinstance(key, tuple):
+            raise ValueError(msg)
+
+        if len(key) != 2:
+            raise ValueError(msg)
+
+        iindexer, cindexer = key
+
+        if iindexer != slice(None):
+            raise ValueError(msg)
+
+        return self._iloc(iindexer, cindexer)
+
+    def _iloc(self, iindexer, cindexer):
+        assert iindexer == slice(None)
+        meta = self._make_meta( iindexer, cindexer)
+
+        return self.obj.map_partitions(methods.iloc, cindexer, meta=meta)
+
+
+class _LocIndexer(_IndexerBase):
+    """ Helper class for the .loc accessor """
+
+    @property
+    def _meta_indexer(self):
+        return self.obj._meta.loc
 
     def __getitem__(self, key):
 
@@ -110,8 +151,8 @@ class _LocIndexer(object):
         else:
             divisions = [None, None]
             dsk = {(name, 0): meta.head(0)}
-        return new_dd_object(merge(self.obj.dask, dsk), name,
-                             meta=meta, divisions=divisions)
+        graph = HighLevelGraph.from_collections(name, dsk, dependencies=[self.obj])
+        return new_dd_object(graph, name, meta=meta, divisions=divisions)
 
     def _loc_element(self, iindexer, cindexer):
         name = 'loc-%s' % tokenize(iindexer, self.obj)
@@ -124,8 +165,8 @@ class _LocIndexer(object):
                            slice(iindexer, iindexer), cindexer)}
 
         meta = self._make_meta(iindexer, cindexer)
-        return new_dd_object(merge(self.obj.dask, dsk), name,
-                             meta=meta, divisions=[iindexer, iindexer])
+        graph = HighLevelGraph.from_collections(name, dsk, dependencies=[self.obj])
+        return new_dd_object(graph, name, meta=meta, divisions=[iindexer, iindexer])
 
     def _get_partitions(self, keys):
         if isinstance(keys, (list, np.ndarray)):
@@ -195,8 +236,8 @@ class _LocIndexer(object):
         assert len(divisions) == len(dsk) + 1
 
         meta = self._make_meta(iindexer, cindexer)
-        return new_dd_object(merge(self.obj.dask, dsk), name,
-                             meta=meta, divisions=divisions)
+        graph = HighLevelGraph.from_collections(name, dsk, dependencies=[self.obj])
+        return new_dd_object(graph, name, meta=meta, divisions=divisions)
 
 
 def _partition_of_index_value(divisions, val):
