@@ -3,9 +3,10 @@ from __future__ import print_function, division, absolute_import
 from .io import (from_array, from_bcolz, from_array, from_bcolz,
                  from_pandas, from_dask_array,
                  from_delayed, dataframe_from_ctable, to_bag, to_records)
-from .csv import read_csv, to_csv, read_table
+from .csv import read_csv, to_csv, read_table, read_fwf
 from .hdf import read_hdf, to_hdf
 from .sql import read_sql_table
+from .json import read_json, to_json
 from . import demo
 try:
     from .parquet import read_parquet, to_parquet
