@@ -52,10 +52,9 @@ df = pd.DataFrame({'x': [i * 7 % 5 for i in range(nrows)],  # Not sorted
 ddf = dd.from_pandas(df, npartitions=npartitions)
 
 
-@pytest.fixture(params=[pytest.mark.skipif(not fastparquet, 'fastparquet',
-                                           reason='fastparquet not found'),
-                        pytest.mark.skipif(not pq, 'pyarrow',
-                                           reason='pyarrow not found')])
+@pytest.fixture(params=[
+    pytest.param('fastparquet', marks=pytest.mark.skipif(not fastparquet, reason='fastparquet not found')),
+    pytest.param('pyarrow', marks=pytest.mark.skipif(not pq, reason='pyarrow not found'))])
 def engine(request):
     return request.param
 
@@ -177,6 +176,10 @@ def test_empty(tmpdir, write_engine, read_engine, index):
 
 @write_read_engines()
 def test_read_glob(tmpdir, write_engine, read_engine):
+    if write_engine == read_engine == 'fastparquet' and os.name == 'nt':
+        # fastparquet or dask is not normalizing filepaths correctly on
+        # windows.
+        pytest.skip("filepath bug.")
     fn = str(tmpdir)
     ddf.to_parquet(fn, engine=write_engine)
     if os.path.exists(os.path.join(fn, '_metadata')):
@@ -199,6 +202,11 @@ def test_read_glob(tmpdir, write_engine, read_engine):
 
 @write_read_engines()
 def test_read_list(tmpdir, write_engine, read_engine):
+    if write_engine == read_engine == 'fastparquet' and os.name == 'nt':
+        # fastparquet or dask is not normalizing filepaths correctly on
+        # windows.
+        pytest.skip("filepath bug.")
+
     tmpdir = str(tmpdir)
     ddf.to_parquet(tmpdir, engine=write_engine)
     files = sorted([os.path.join(tmpdir, f)
@@ -433,6 +441,8 @@ def test_names(tmpdir, engine):
             set(read(fn, columns=['x']).dask))
 
 
+@pytest.mark.xfail(reason="parquet column fusion is special cased today"
+                          " we'll need to find a more general solution near-term")
 @pytest.mark.parametrize('c', [['x'], 'x', ['x', 'y'], []])
 def test_optimize(tmpdir, c):
     check_fastparquet()
@@ -546,7 +556,7 @@ def test_append_with_partition(tmpdir):
                   ignore_divisions=True)
 
     out = dd.read_parquet(tmp).compute()
-    out['lon'] = out.lon.astype('int64')  # just to pass assert
+    out['lon'] = out.lon.astype('int')  # just to pass assert
     # sort required since partitioning breaks index order
     assert_eq(out.sort_values('value'), pd.concat([df0, df1])[out.columns],
               check_index=False)
@@ -709,8 +719,9 @@ def test_read_parquet_custom_columns(tmpdir, engine):
     (pd.DataFrame({'x': pd.Categorical([1, 2, 1])}), {}, {'categories': ['x']}),
     (pd.DataFrame({'x': list(map(pd.Timestamp, [3000, 2000, 1000]))}), {}, {}),
     (pd.DataFrame({'x': [3000, 2000, 1000]}).astype('M8[ns]'), {}, {}),
-    pytest.mark.xfail((pd.DataFrame({'x': [3, 2, 1]}).astype('M8[ns]'), {}, {}),
-                      reason="Parquet doesn't support nanosecond precision"),
+    pytest.param(pd.DataFrame({'x': [3, 2, 1]}).astype('M8[ns]'), {}, {},
+                 marks=pytest.mark.xfail(
+                     reason="Parquet doesn't support nanosecond precision")),
     (pd.DataFrame({'x': [3, 2, 1]}).astype('M8[us]'), {}, {}),
     (pd.DataFrame({'x': [3, 2, 1]}).astype('M8[ms]'), {}, {}),
     (pd.DataFrame({'x': [3, 2, 1]}).astype('uint16'), {}, {}),
@@ -1261,6 +1272,7 @@ def test_writing_parquet_with_kwargs(tmpdir, engine):
     fn = str(tmpdir)
     path1 = os.path.join(fn, 'normal')
     path2 = os.path.join(fn, 'partitioned')
+    pytest.importorskip("snappy")
 
     df = pd.DataFrame({'a': np.random.choice(['A', 'B', 'C'], size=100),
                        'b': np.random.random(size=100),
@@ -1301,6 +1313,7 @@ def test_writing_parquet_with_unknown_kwargs(tmpdir, engine):
 
 
 def test_select_partitioned_column(tmpdir, engine):
+    pytest.importorskip("snappy")
     if engine == 'pyarrow':
         import pyarrow as pa
         if pa.__version__ < LooseVersion('0.9.0'):
@@ -1345,3 +1358,34 @@ def test_informative_error_messages():
     assert 'foo' in str(info.value)
     assert 'arrow' in str(info.value)
     assert 'fastparquet' in str(info.value)
+
+
+def test_append_cat_fp(tmpdir):
+    pytest.importorskip('fastparquet')
+    path = str(tmpdir)
+    # https://github.com/dask/dask/issues/4120
+    df = pd.DataFrame({"x": ["a", "a", "b", "a", "b"]})
+    df["x"] = df["x"].astype("category")
+    ddf = dd.from_pandas(df, npartitions=1)
+
+    dd.to_parquet(ddf, path)
+
+    # this fails:
+    dd.to_parquet(ddf, path, append=True, ignore_divisions=True)
+    d = dd.read_parquet(path).compute()
+    assert d['x'].tolist() == ["a", "a", "b", "a", "b"] * 2
+
+
+def test_passing_parquetfile(tmpdir):
+    import shutil
+    fp = pytest.importorskip('fastparquet')
+    path = str(tmpdir)
+    df = pd.DataFrame({"x": [1, 3, 2, 4]})
+    ddf = dd.from_pandas(df, npartitions=1)
+
+    dd.to_parquet(ddf, path)
+    pf = fp.ParquetFile(path)
+    shutil.rmtree(path)
+
+    # should pass, because no need to re-read metadata
+    dd.read_parquet(pf)
