@@ -11,20 +11,43 @@ from contextlib import contextmanager
 import numpy as np
 import pandas as pd
 import pandas.util.testing as tm
-from pandas.api.types import is_categorical_dtype, is_scalar
+from pandas.api.types import (
+    is_categorical_dtype, is_scalar, is_sparse, is_period_dtype,
+)
 try:
     from pandas.api.types import is_datetime64tz_dtype
 except ImportError:
     # pandas < 0.19.2
     from pandas.core.common import is_datetime64tz_dtype
 
-from ..compatibility import PY2, Iterator
+try:
+    from pandas.api.types import is_interval_dtype
+except ImportError:
+    is_interval_dtype = lambda dtype: False
+
+from .extensions import make_array_nonempty
+from ..base import is_dask_collection
+from ..compatibility import PY2, Iterator, Mapping
 from ..core import get_deps
 from ..local import get_sync
 from ..utils import asciitable, is_arraylike, Dispatch
 
 
 PANDAS_VERSION = LooseVersion(pd.__version__)
+PANDAS_GT_0240 = PANDAS_VERSION >= LooseVersion("0.24.0rc1")
+HAS_INT_NA = PANDAS_GT_0240
+
+
+def is_integer_na_dtype(t):
+    dtype = getattr(t, 'dtype', t)
+    if HAS_INT_NA:
+        types = (
+            pd.Int8Dtype, pd.Int16Dtype, pd.Int32Dtype, pd.Int64Dtype,
+            pd.UInt8Dtype, pd.UInt16Dtype, pd.UInt32Dtype, pd.UInt64Dtype,
+        )
+    else:
+        types = ()
+    return isinstance(dtype, types)
 
 
 def shard_df_on_index(df, divisions):
@@ -427,6 +450,7 @@ def _nonempty_scalar(x):
 
 @meta_nonempty.register(pd.Series)
 def _nonempty_series(s, idx=None):
+    # TODO: Use register dtypes with make_array_nonempty
     if idx is None:
         idx = _nonempty_index(s.index)
     dtype = s.dtype
@@ -442,6 +466,28 @@ def _nonempty_series(s, idx=None):
             cats = None
         data = pd.Categorical(data, categories=cats,
                               ordered=s.cat.ordered)
+    elif is_integer_na_dtype(dtype):
+        data = pd.array([1, None], dtype=dtype)
+    elif is_period_dtype(dtype):
+        # pandas 0.24.0+ should infer this to be Series[Period[freq]]
+        freq = dtype.freq
+        data = [pd.Period('2000', freq), pd.Period('2001', freq)]
+    elif is_sparse(dtype):
+        # TODO: pandas <0.24
+        # Pandas <= 0.23.4:
+        if PANDAS_GT_0240:
+            entry = _scalar_from_dtype(dtype.subtype)
+        else:
+            entry = _scalar_from_dtype(dtype.subtype)
+        data = pd.SparseArray([entry, entry], dtype=dtype)
+    elif is_interval_dtype(dtype):
+        entry = _scalar_from_dtype(dtype.subtype)
+        if PANDAS_GT_0240:
+            data = pd.array([entry, entry], dtype=dtype)
+        else:
+            data = np.array([entry, entry], dtype=dtype)
+    elif type(dtype) in make_array_nonempty._lookup:
+        data = make_array_nonempty(dtype)
     else:
         entry = _scalar_from_dtype(dtype)
         data = np.array([entry, entry], dtype=dtype)
@@ -449,6 +495,22 @@ def _nonempty_series(s, idx=None):
     return pd.Series(data, name=s.name, index=idx)
 
 
+def is_dataframe_like(df):
+    """ Looks like a Pandas DataFrame """
+    return set(dir(df)) > {'dtypes', 'columns', 'groupby', 'head'}
+
+
+def is_series_like(s):
+    """ Looks like a Pandas Series """
+    return set(dir(s)) > {'name', 'dtype', 'groupby', 'head'}
+
+
+def is_index_like(s):
+    """ Looks like a Pandas Index """
+    attrs = set(dir(s))
+    return attrs > {'name', 'dtype'} and 'head' not in attrs
+
+
 def check_meta(x, meta, funcname=None, numeric_equal=True):
     """Check that the dask metadata matches the result.
 
@@ -485,14 +547,11 @@ def check_meta(x, meta, funcname=None, numeric_equal=True):
             return a == b
         return (a.kind in eq_types and b.kind in eq_types) or (a == b)
 
-    from .core import parallel_types
-    if not isinstance(meta, parallel_types()):
+    if (not (is_dataframe_like(meta) or is_series_like(meta) or is_index_like(meta))
+            or is_dask_collection(meta)):
         raise TypeError("Expected partition to be DataFrame, Series, or "
                         "Index, got `%s`" % type(meta).__name__)
 
-    def is_dataframe_like(df):
-        return all(hasattr(df, attr) for attr in ['dtypes', 'columns', 'groupby', 'head'])
-
     if type(x) != type(meta):
         errmsg = ("Expected partition of type `%s` but got "
                   "`%s`" % (type(meta).__name__, type(x).__name__))
@@ -501,13 +560,20 @@ def check_meta(x, meta, funcname=None, numeric_equal=True):
         if PANDAS_VERSION >= LooseVersion('0.23.0'):
             kwargs['sort'] = True
         dtypes = pd.concat([x.dtypes, meta.dtypes], axis=1, **kwargs)
-        bad = [(col, a, b) for col, a, b in dtypes.fillna('-').itertuples()
-               if not equal_dtypes(a, b)]
-        if not bad:
+        bad_dtypes = [(col, a, b) for col, a, b in dtypes.fillna('-').itertuples()
+                      if not equal_dtypes(a, b)]
+        if bad_dtypes:
+            errmsg = ("Partition type: `%s`\n%s" %
+                      (type(meta).__name__,
+                       asciitable(['Column', 'Found', 'Expected'], bad_dtypes)))
+        elif not np.array_equal(np.nan_to_num(meta.columns),
+                                np.nan_to_num(x.columns)):
+            errmsg = ("The columns in the computed data do not match"
+                      " the columns in the provided metadata.\n"
+                      " %s\n  :%s" %
+                      (meta.columns, x.columns))
+        else:
             return x
-        errmsg = ("Partition type: `%s`\n%s" %
-                  (type(meta).__name__,
-                   asciitable(['Column', 'Found', 'Expected'], bad)))
     else:
         if equal_dtypes(x.dtype, meta.dtype):
             return x
@@ -594,6 +660,9 @@ def _maybe_sort(a):
     # sort by value, then index
     try:
         if isinstance(a, pd.DataFrame):
+            if set(a.index.names) & set(a.columns):
+                a.index.names = ['-overlapped-index-name-%d' % i
+                                 for i in range(len(a.index.names))]
             a = a.sort_values(by=a.columns.tolist())
         else:
             a = a.sort_values()
@@ -646,7 +715,7 @@ def assert_eq(a, b, check_names=True, check_dtypes=True,
 def assert_dask_graph(dask, label):
     if hasattr(dask, 'dask'):
         dask = dask.dask
-    assert isinstance(dask, dict)
+    assert isinstance(dask, Mapping)
     for k in dask:
         if isinstance(k, tuple):
             k = k[0]
